package progressiveClientServer.gettingBackObject;


import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import com.google.gson.Gson;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;

public class ClientCommunicator {
	private static final Gson gson = new Gson();
	
//Singleton
	public static ClientCommunicator SINGLETON = new ClientCommunicator();
	
//Attributes
	
//Constructors
	public ClientCommunicator() {}
		
//Commands
	public Object send() {
		HttpURLConnection connection = openConnection(COMMAND_HANDLER_DESIGNATOR, HTTP_GET, false);
		//sendToServerCommunicator(connection, null); //Not needed, we are not sending anything
		Object result = getResult(connection, Person.class); 

		return result;
	}
	
	public static void main(String[] args) {
		Person person = (Person)ClientCommunicator.SINGLETON.send();
		System.out.println("Person = " + person);
	}
		
//Auxiliary Constants, Attributes, and Methods
	private static final String SERVER_HOST = "localhost";
	private static final String URL_PREFIX = "http://" + SERVER_HOST + ":" + ServerCommunicator.getServerPortNumber();
	private static final String HTTP_POST = "POST";
	private static final String HTTP_GET = "GET";
	private static final String COMMAND_HANDLER_DESIGNATOR = "/";
	
	private HttpURLConnection openConnection(String contextIdentifier,
											 String requestMethod,
            								 boolean sendingSomthingToServer)
	{
		HttpURLConnection result = null;
		try {
			URL url = new URL(URL_PREFIX + contextIdentifier);
			result = (HttpURLConnection)url.openConnection();
			result.setRequestMethod(requestMethod);
			result.setDoOutput(sendingSomthingToServer);
			result.connect();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return result;
	}
	
	private void sendToServerCommunicator(HttpURLConnection connection,
            							  Object objectToSend)
	{
		//Not sending anything
	}
	
	private Object getResult(HttpURLConnection connection, Class<?> klass) {
		Object result = null;
		try {
			if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
				//0 means the body response length == 0; it was empty
				if(connection.getContentLength() == 0) {
					System.out.println("Response body was empty");
				} else if(connection.getContentLength() == -1) {
					//-1 means the body was not empty but has an unknown about of information
					InputStreamReader inputStreamReader = new InputStreamReader(connection.getInputStream());
					/******************************************/
					result = gson.fromJson(inputStreamReader, klass);
					/******************************************/
					inputStreamReader.close();
				}
			} else {
				throw new Exception(String.format("http code %d",	connection.getResponseCode()));
			}
		} catch (JsonSyntaxException | JsonIOException | IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return result;
	}
}
