package progressiveClientServer.sendProxy;

import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;

import progressiveClientServer.sendProxy.services.DoubleValueService;
import progressiveClientServer.sendProxy.services.NegateValueService;
import progressiveClientServer.sendingObjects.ClientCommunicator;

import com.google.gson.Gson;
import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

public class ServerCommunicator {

	private static final int SERVER_PORT_NUMBER = 8080;
	private static final int MAX_WAITING_CONNECTIONS = 10;
	private static Gson gson = new Gson();
	
	private HttpServer server;
	
	private ServerCommunicator() {}

	private void run() {

		try {
			server = HttpServer.create(new InetSocketAddress(SERVER_PORT_NUMBER),
									   MAX_WAITING_CONNECTIONS);
		} 
		catch (IOException e) {
			System.out.println("Could not create HTTP server: " + e.getMessage());
			e.printStackTrace();
			return;
		}

		server.setExecutor(null); // use the default executor
		
		server.createContext(DOUBLE_VALUE_DESIGNATOR, doubleValueHandler);
		server.createContext(NEGATE_VALUE_DESIGNATOR, negateValueHandler);

		
		server.start();
	}
	
	private HttpHandler doubleValueHandler = new HttpHandler() {
		@Override
		public void handle(HttpExchange exchange) throws IOException {
			Headers headers = exchange.getRequestHeaders();
			String authorizationCode = headers.getFirst(ClientCommunicator.AUTHORIZATION_KEY);
			System.out.println("The authorization code in doubleValueHandler = " + authorizationCode);	
			
			InputStreamReader inputStreamReader = new InputStreamReader(exchange.getRequestBody());
			int intValue = (int)gson.fromJson(inputStreamReader, int.class);
			inputStreamReader.close();
			DoubleValueService.SINGLETON.doubleValue(intValue);
			exchange.getRequestBody().close();
			
			//-1 means the response body is empty
			exchange.sendResponseHeaders(HttpURLConnection.HTTP_OK, -1);
		}
	};	
	
	private HttpHandler negateValueHandler = new HttpHandler() {
		@Override
		public void handle(HttpExchange exchange) throws IOException {	
			Headers headers = exchange.getRequestHeaders();
			String authorizationCode = headers.getFirst(ClientCommunicator.AUTHORIZATION_KEY);
			System.out.println("The authorization code in negateValueHandler = " + authorizationCode);	
			
			InputStreamReader inputStreamReader = new InputStreamReader(exchange.getRequestBody());
			double doubleValue = (double)gson.fromJson(inputStreamReader, double.class);
			inputStreamReader.close();
			
			NegateValueService.SINGLETON.negateValue(doubleValue);
			
			exchange.getRequestBody().close();
			
			//-1 means the response body is empty
			exchange.sendResponseHeaders(HttpURLConnection.HTTP_OK, -1);
		}
	};
	
	public static int getServerPortNumber() {
		return SERVER_PORT_NUMBER;
	}
	
	public static void main(String[] args) {
		new ServerCommunicator().run();
	}
	
	public static final String DOUBLE_VALUE_DESIGNATOR = "/double";
	public static final String NEGATE_VALUE_DESIGNATOR = "/negate";

}
